/* eslint-disable */
"use strict";
const path = require("path");
const fs = require("fs");
const Generator = require("yeoman-generator");
const chalk = require("chalk");
const yosay = require("yosay");

module.exports = class extends Generator {
  prompting() {
    // Have Yeoman greet the user.
    this.log(
      yosay(
        `Welcome to the neat ${chalk.red("generator-vue-template")} generator!`
      )
    );

    const prompts = [
      {
        type: "input",
        name: "name",
        message: "Your project name(请输入你的项目名)",
        default: this.appname
      },
      {
        type: "input",
        name: "description",
        required: false,
        message: "Project description",
        default: "A ProductAI vue project."
      },
      {
        type: "checkbox",
        name: "choices",
        message: "Choose your modules(请选择所需模块)",
        choices: [
          "Typescript",
          "vuex"
        ],
        filter: function(val) {
          // 使用filter将回答变为小写
          return val.map(item => item.toLowerCase());
        }
      },
      {
        type: "list",
        name: "http",
        message: 'Choose a http request way',
        choices: ["fetch", "axios"]
      },
      {
        type: "confirm",
        name: "someAnswer",
        message: "Would you like to enable this option?",
        default: true
      }
    ];

    return this.prompt(prompts).then(props => {
      // To access props later use this.props.someAnswer;
      this.props = props;
    });
  }

  writing() {
    this.initPackage();
    this.writeMapper();
    this.renderFile();
  }

  install() {
    // this.spawnCommandSync(`cd ${this.destinationPath(`./${this.props.name}`)}`)
    this.npmInstall()
  }

  initPackage() {
    let pkg = this.fs.readJSON(this.templatePath("package.json"), {});
    let pkgLock = this.fs.readJSON(this.templatePath("package-lock.json"), {});
    const { props } = this;
    const { name, description, choices, http } = props;
    pkg.name = pkgLock.name = name;
    pkg.description = description;

    if (http !== "axios") {
      delete pkg.dependencies.axios;
    }
    if (choices.indexOf("vuex") === -1) {
      delete pkg.dependencies.vuex;
    }

    this.fs.writeJSON(this.destinationPath(`./package.json`), pkg);
    this.fs.writeJSON(
      this.destinationPath(`./package-lock.json`),
      pkgLock
    );
  }

  writeMainJs() {
    const { name, choices } = this.props;
    let mainJs = this.fs.read(this.templatePath("./src/main.js"));
    if (choices.indexOf("vuex") === -1) {
      mainJs = mainJs.replace(`import store from './store'`, "");
      mainJs = mainJs.replace(`store,`, "");
    }
    this.fs.write(this.destinationPath(`./src/main.js`), mainJs);
  }

  writeMapper() {
    const { name, choices, http } = this.props;
    if(!http || choices.indexOf('vuex') == -1) return

    let mapper = ''
    if(http === 'axios') {
      mapper = this.fs.read(this.templatePath("./src/store/mapper-axios.js"));
    }else{
      mapper = this.fs.read(this.templatePath("./src/store/mapper-fetch.js"));
    }
    this.fs.write(this.destinationPath(`./src/store/mapper.js`), mapper);
  }

  renderFile() {
    let target = [];
    const files = fs.readdirSync(this.templatePath("./"));
    const blackList = ["main.js", "mapper-axios.js", "mapper-fetch.js"];
    const { choices, http } = this.props;
    if (choices.indexOf("vuex") === -1) {
      blackList.push("store");
    }
    if(http === 'axios') {
      blackList.push('fetch.js')
    }else if(http === 'fetch') {
      blackList.push('axios.js')
    }else {
      blackList.push('http')
    }

    const getFiles = filePath => {
      let allFiles = [];
      const files = fs.readdirSync(filePath);
      const root = this.templatePath("./");
      files.forEach(item => {
        if (blackList.indexOf(item) === -1) {
          let stat = fs.lstatSync(filePath + "/" + item);
          if (stat.isDirectory()) {
            allFiles = [
              ...allFiles,
              ...getFiles(path.join(filePath, "./", item))
            ];
          } else if (item !== "package.json" && item !== "package-lock.json") {
            const file = path
              .join(filePath, "./", item)
              .replace(root, "")
              .replace(/\\/g, "/");
            this.fs.copyTpl(
              this.templatePath(`./${file}`),
              this.destinationPath(`./${file}`)
            );
            allFiles.push(file);
          }
        }
      });
      return allFiles;
    };

    target = getFiles(this.templatePath("./"));
    this.writeMainJs();
  }
};
