const path = require('path')
const VueLoaderPlugin = require('vue-loader/lib/plugin')
const HtmlWebpackPlugin = require('html-webpack-plugin')

module.exports = {
  mode: process.env.NODE_ENV,
  // 输入
  entry: {
    app: path.join(__dirname, '../src/main.js')
  },

  // 输出
  output: {
    path: path.join(__dirname, '../dist'),
    filename: 'bundle.js',
    publicPath: process.env.NODE_ENV === 'production' ? './' : '/'
  },

  // 文件重定向
  resolve: {
    alias: {
      'vue$': 'vue/dist/vue.esm.js', //vue默认使用runtime,没有complier功能,需要重定向到vue.esm.js编译vue文件
      '@': path.join(__dirname, '../src')
    },
    extensions: ['.js', '.jsx', '.json']  //在导入的文件没有后缀时,会自动带上此数组数据查找文件
  },

  module: {
    rules: [
      {
        test: /\.vue$/,
        use: 'vue-loader'
      },
      {
        test: /\.(png|jpg|jepg|svg)$/,
        use: [
          {
            loader: 'url-loader',
            options: {
              esModule: false,
              limit: 5 * 1024,  // 5kb以下的图片会转成base64
              name: 'static/image/[name][hash].[ext]'  //打包输出路径
            }
          }
        ]
      },
      {
        test: /\.js$/,
        use: [
          {
            loader: 'babel-loader'
          }
        ],
        include: [
          path.join(__dirname, '../src')
        ]
      }
    ]
  },

  plugins: [  //需要配置了插件才能使用相应的loader
    new VueLoaderPlugin(),
    new HtmlWebpackPlugin({
      template: './public/index.html', //打包到dist文件夹的html文件模板路径 因为使用了file-loader，所以直接静态资源路径做根路径
      inject: true, //script插入的标签
      manify: {
        removeComment: true //删除注释
      }
    })
  ]
}