const base = require('./webpack.config.base')
const merge = require('webpack-merge')
const webpack = require('webpack')
const path = require('path')

module.exports = merge(base, {
  devtool: 'inline-source-map',  // 设置生成的代码的格式,可以方便调试
  devServer: {
    clientLogLevel: 'none',
    contentBase: path.join(__dirname, '../public'), //告诉服务器从哪里提供内容（默认当前工作目录），用于指定被访问的html文件的位置
    //lazy: true, /* 仅在请求时会进行编译 */
    filename:"bundle.js",
    port: 8080,
    host: 'localhost',
    open: true, //自动打开浏览器
    hot: true, //模块热替换
    historyApiFallback: true, //当使用html5 history api,将会在响应404时返回index.html 或自定义一个对象设置返回页面
    overlay: {
      errors: true    //出错时全屏显示
    },
    compress: true, //开启gzip压缩
    stats: {  // 设置控制台显示信息
      chunks: false,
      children: false,
      modules: false,
      entrypoints: false,
      warnings: false,
      performance: false,  // 是否输出webpack建议（如文件体积大小）
      version: false,
      hash: false,
      assets: false
    },
    proxy: {
      '/api-dev': {
        target: '',
        changeOrigin: true,
        pathRewrite: {
          '^/api-dev': ''
        }
      }
    },
    headers: {  //将所有请求都加上此header信息
      // token: ''
    }
  },

  module: {
    rules: [
      {
        test: /.css$/,
        use: ['style-loader', 'css-loader']
      }
    ]
  },

  plugins: [
    new webpack.HotModuleReplacementPlugin()
  ]
})