import axios from 'axios'
import factory from '@/http/factory'

const options = {
    baseURL: '',
    timeout: 10000
}

const instance = axios.create(options)

instance.interceptors.request.use(config => {
    /** do something before request */
    
    return config
}, error => {
    return Promise.reject(error);
})

instance.interceptors.response.use(response => {
    /** do something before response returned */
    const { data } = response

    return data
}, error => {
    return Promise.reject(error);
})

export default instance

function request({ service = '', serviceArguments = {} }) {
    const [moduleName, serviceName] = service.split('/');
    const api = factory({ moduleName, serviceName, serviceArguments })
    const { url, options } = api
    const { headers, body, method } = options
    return instance({
        url,
        headers: headers,
        method: method,
        data: body
    })
}

export const XHR = new Proxy(
    {},
    {
      get(target, service) {
        // service = 'user/search'
        return (serviceArguments) => request({ service, serviceArguments });
      },
    }
  );