import factory from './factory';

// 生成http请求
function serve({ service = '', serviceArguments = {} }) {
  const [moduleName, serviceName] = service.split('/');
  let { url, options } = factory({ moduleName, serviceName, serviceArguments });

  const controller = new AbortController(); // h5 API 用于终止网络请求的控制器
//   const signal = controller.signal;  // 可传入到fetch变成手动abort请求

  return new Promise((resolve, reject) => {
    TIMEOUT_GUARD({ reject, controller, url });
    fetch(url, { ...options })  // 发送请求
      .then((res) => {
        const { ok, status, statusText } = res;
        if (ok) {
          return res.json().then((res) => {
            const { code } = res;
            if (code === 200) {
                resolve(res)
                return
            } else {
              // 第三级错误：接口返回错误
              reject({
                code: 9001,
                msg: `${service}--接口返回错误`,
                error: res,
              });
              return
            }
          });
        } else {
          // 第二级错误：远程服务器错误
          return {
            code: 8001,
            msg: '远程服务器错误',
            error: { status, statusText },
          };
        }
      })
      .catch((error) => {
        // 第一级错误：网络错误（超时或断网）
        // 如果使用第一种超时实现方式，则会有这个错误
        console.error(error);
        if (error.name === 'AbortError') {
          return {
            code: 7001,
            error,
          };
        }
        // 如果使用第二种超时实现方式，则直接返回错误，不再包装
        if (error.code === 7002) return error;
        return {
          code: 7000,
          error,
        };
      }); 
  });
}

// 定义超时拒绝约定（promise）的辅助函数
function TIMEOUT_GUARD({ reject, controller, url }) {
  setTimeout(() => {
    reject({
      code: 7002,
      error: new Error(`${url}请求超时`),
    });
    controller.abort(); // abort后会执行.catch
  }, 10000);    // 设置超时时长
}

export default new Proxy(
  {},
  {
    get(target, service) {
      // service = 'user/search'
      return (serviceArguments) => serve({ service, serviceArguments });
    },
  }
);

// 不支持proxy时使用
// export default Object.keys(API).reduce((pre, service) => {
//     pre[service] = serviceArguments => serve({service, serviceArguments})
//     return pre
// }, {})
